@@ -182,6 +182,12 @@ static int readStanza(char ** cfg, struct pumpOverrideInfo * overrideList,
 		return 1;
 	    }
 	    override->flags |= OVERRIDE_FLAG_NODNS;
+	} else if (!strcmp(start, "nobootp")) {
+	    if (*rest) {
+		parseError(*lineNum, "unexpected argument to nobootp directive");
+		return 1;
+	    }
+	    override->flags |= OVERRIDE_FLAG_NOBOOTP;
 	} else if (!strcmp(start, "nogateway")) {
 	    if (*rest) {
 		parseError(*lineNum, "unexpected argument to nogateway directive");
