@@ -67,6 +67,7 @@ typedef short bp_int16;
 
 #define DHCP_OPTION_LOGSRVS		7
 #define DHCP_OPTION_LPRSRVS		9
+#define DHCP_OPTION_MTU			26
 #define DHCP_OPTION_NTPSRVS		42
 #define DHCP_OPTION_XFNTSRVS		48
 #define DHCP_OPTION_XDMSRVS		49
@@ -115,7 +116,7 @@ struct bootpRequest {
     char hwaddr[16];
     char servername[64];
     char bootfile[128];
-    char vendor[DHCP_VENDOR_LENGTH];
+    unsigned char vendor[DHCP_VENDOR_LENGTH];
 } ;
 
 struct psuedohUdpHeader {
@@ -230,7 +231,6 @@ char * pumpDisableInterface(char * device) {
 }
 
 char * pumpSetupInterface(struct pumpNetIntf * intf) {
-    char * rc;
     struct sockaddr_in * addrp;
     struct ifreq req;
     struct rtentry route;
@@ -257,6 +257,12 @@ char * pumpSetupInterface(struct pumpNetIntf * intf) {
     if (ioctl(s, SIOCSIFBRDADDR, &req))
 	return perrorstr("SIOCSIFBRDADDR");
 
+    if (intf->set & PUMP_INTFINFO_HAS_MTU) {
+        req.ifr_mtu = intf->mtu;
+        if (ioctl(s, SIOCSIFMTU, &req))
+            return perrorstr("SIOCSIFMTU");
+    }
+
     /* Bring up the device, and specifically allow broadcasts through it.
        Don't mess with flags we don't understand though. */
     if (ioctl(s, SIOCGIFFLAGS, &req))
@@ -596,6 +602,16 @@ static void parseReply(struct bootpRequest * breq, struct pumpNetIntf * intf) {
 	    case DHCP_OPTION_OVERLOAD:
 		/* FIXME: we should pay attention to this */
 		break;
+
+	    case DHCP_OPTION_MTU:
+                {
+                uint16_t us;
+                memcpy(&us, chptr, 2);
+		intf->mtu = (int)ntohs(us);
+		intf->set |= PUMP_INTFINFO_HAS_MTU;
+		syslog (LOG_DEBUG, "intf: mtu: %i", intf->mtu);
+                }
+		break;
 	}
 
 	chptr += length;
@@ -704,7 +720,8 @@ void debugbootpRequest(char *name, struct bootpRequest *breq)  {
     syslog (LOG_DEBUG, "%s: bootfile: %s", name, breq->bootfile);
     
     vndptr = breq->vendor;
-    sprintf (vendor, "0x%02x 0x%02x 0x%02x 0x%02x", *vndptr++, *vndptr++, *vndptr++, *vndptr++);
+    sprintf (vendor, "0x%02x 0x%02x 0x%02x 0x%02x", vndptr[0], vndptr[1], vndptr[2], vndptr[3]);
+    vndptr += 4;
     syslog (LOG_DEBUG, "%s: vendor: %s", name, vendor);
     
     
@@ -756,10 +773,11 @@ static char * handleTransaction(int s, struct pumpOverrideInfo * override,
     fd_set readfs;
     int i, j;
     struct sockaddr_pkt tmpAddress;
+    socklen_t socksize;
     int gotit = 0;
     int tries;
     int nextTimeout = 2;
-    time_t timeoutTime;
+    time_t timeoutTime = 0;
     int sin;
     int resend = 1;
     struct ethhdr;
@@ -834,9 +852,10 @@ static char * handleTransaction(int s, struct pumpOverrideInfo * override,
 	    break;
 
 	  case 1:
-	    i = sizeof(tmpAddress);
+            socksize = sizeof(tmpAddress);
 	    if ((j = recvfrom(sin, ethPacket, sizeof(ethPacket), 0, 
-		     (struct sockaddr *) &tmpAddress, &i)) < 0)
+                              (struct sockaddr *) &tmpAddress, 
+                              &socksize)) < 0)
 		return perrorstr("recvfrom");
 
 	    /* We need to do some basic sanity checking of the header */
@@ -1003,7 +1022,7 @@ static int createSocket(const char * device) {
     }
 
     if (setsockopt(s, SOL_SOCKET, SO_BINDTODEVICE, device, strlen(device)+1)) {
-	syslog(LOG_ERR, "SO_BINDTODEVICE %s (%d) failed: %s", device, strlen(device), strerror(errno));
+	syslog(LOG_ERR, "SO_BINDTODEVICE %s (%zd) failed: %s", device, strlen(device), strerror(errno));
     }
 
     memset(&clientAddr.sin_addr, 0, sizeof(&clientAddr.sin_addr));
@@ -1204,6 +1223,7 @@ static void buildRequest(struct bootpRequest * req, int flags, int type,
     optionsRequested[numOptions++] = DHCP_OPTION_NTPSRVS;
     optionsRequested[numOptions++] = DHCP_OPTION_XFNTSRVS;
     optionsRequested[numOptions++] = DHCP_OPTION_XDMSRVS;
+    optionsRequested[numOptions++] = DHCP_OPTION_MTU;
     addVendorCode(req, DHCP_OPTION_OPTIONREQ, numOptions, 
 		  optionsRequested);
 
@@ -1237,7 +1257,7 @@ char * pumpDhcpClassRun(char * device, int flags, int reqLease,
     struct sockaddr_in clientAddr;
     struct sockaddr_in broadcastAddr;
     struct bootpRequest breq, bresp;
-    unsigned char * chptr;
+    char * chptr;
     time_t startTime = pumpUptime();
     int true = 1;
     int ttl = 16;
@@ -1423,8 +1443,8 @@ char * pumpDhcpClassRun(char * device, int flags, int reqLease,
 char * pumpDhcpRun(char * device, int flags, int reqLease,
                   char * reqHostname, struct pumpNetIntf * intf,
                   struct pumpOverrideInfo * override) {
-    pumpDhcpClassRun(device, flags, reqLease, reqHostname, NULL, intf,
-                     override);
+    return pumpDhcpClassRun(device, flags, reqLease, reqHostname, NULL, intf,
+                            override);
 }
 
 void pumpInitOverride(struct pumpOverrideInfo * override) {
@@ -1461,7 +1481,7 @@ time_t pumpUptime() {
 	return time(NULL);
     }
 
-    fscanf(fd, "%ld", &secs);
+    if (fscanf(fd, "%ld", &secs) != 1) syslog (LOG_DEBUG, "uptime didn't match expected format");
     fclose(fd);
 
     return (time_t)secs;
