@@ -58,7 +58,7 @@
 
 struct intfconfig_s {
     newtComponent ipEntry, nmEntry, gwEntry, nsEntry;
-    char * ip, * nm, * gw, * ns;
+    const char * ip, * nm, * gw, * ns;
 };
 
 typedef int int32;
@@ -259,10 +259,11 @@ static void dhcpBoxCallback(newtComponent co, void * ptr) {
     newtEntrySetFlags(c->nsEntry, NEWT_FLAG_DISABLED, NEWT_FLAGS_TOGGLE);
 }
 
+#ifndef __STANDALONE__
 static int getDnsServers(struct networkDeviceConfig * cfg) {
     int rc;
     char * ns = "";
-    struct newtWinEntry entry[] = { { N_("Nameserver IP"), &ns, 0 },
+    struct newtWinEntry entry[] = { { N_("Nameserver IP"), (const char **)&ns, 0 },
 				      { NULL, NULL, 0 } };
 
     do {
@@ -289,6 +290,7 @@ static int getDnsServers(struct networkDeviceConfig * cfg) {
 
     return LOADER_OK;
 }
+#endif
 
 int readNetConfig(char * device, struct networkDeviceConfig * cfg, int flags) {
     newtComponent text, f, okay, back, answer, dhcpCheckbox;
