@@ -67,7 +67,7 @@ struct command {
 	struct {
 	    char device[20];
 	    int flags;
-	    int reqLease;			/* in hours */
+	    int reqLease;			/* in seconds */
 	    char reqHostname[200];
 	} start;
 	int result;				/* 0 for success */
@@ -310,7 +310,7 @@ static void callScript(char* script,int msg,struct pumpNetIntf* intf) {
     pid_t child;
     char * argv[20];
     char ** nextArg;
-    char * class, * chptr;
+    char * class = NULL, * chptr;
 
     if (!script) return;
 
@@ -360,7 +360,7 @@ static void callScript(char* script,int msg,struct pumpNetIntf* intf) {
 static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * overrides) {
     int conn;
     struct sockaddr_un addr;
-    int addrLength = sizeof(struct sockaddr_un);
+    socklen_t addrLength = sizeof(struct sockaddr_un);
     struct command cmd;
     struct pumpNetIntf intf[20];
     int numInterfaces = 0;
@@ -368,7 +368,7 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
     int closest;
     struct timeval tv;
     fd_set fds;
-    struct pumpOverrideInfo emptyOverride, * o;
+    struct pumpOverrideInfo emptyOverride, * o = NULL;
 
     if (!overrides)
         readPumpConfig(configFile, &overrides);
@@ -386,7 +386,36 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 	closest = -1;
 	if (numInterfaces) {
 	    for (i = 0; i < numInterfaces; i++)
-		if ((intf[i].set & PUMP_INTFINFO_HAS_LEASE) && 
+		/* if this interface has an expired lease due to
+		 * renewal failures and it's time to try again to
+		 * get a new lease, then try again
+		 *
+		 * note: this trys every 30 secs FOREVER; this may
+		 * or may not be desirable.  could also have a back-off
+		 * hueristic that increases the retry delay after each
+		 * failed attempt and a maximum number of tries or
+		 * maximum period of time to try for.
+		 */
+		if ((intf[i].set & PUMP_INTFINFO_NEEDS_NEWLEASE) &&
+		   (intf[i].renewAt < pumpUptime())) {
+		    if (pumpDhcpRun(intf[i].device, 0, 
+			  intf[i].reqLease,
+			  intf[i].set & PUMP_NETINFO_HAS_HOSTNAME
+			    ? intf[i].hostname : NULL,
+			  intf + i, overrides)) {
+
+			    /* failed to get a new lease, so try
+			     * again in 30 seconds
+                             */
+			    intf[i].renewAt = pumpUptime() + 30;
+
+		    } else {
+			intf[i].set &= ~PUMP_INTFINFO_NEEDS_NEWLEASE;
+			callScript(overrides->script, PUMP_SCRIPT_NEWLEASE,
+				   &intf[i]);
+                    }
+		}
+		else if ((intf[i].set & PUMP_INTFINFO_HAS_LEASE) && 
 			(closest == -1 || 
 			       (intf[closest].renewAt > intf[i].renewAt)))
 		    closest = i;
@@ -398,7 +427,12 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 				"failed to renew lease for device %s",
 				intf[closest].device);
 
-			if ((intf[closest].renewAt += pumpUptime() + 30) >
+			/* if the renewal failed, then set renewAt to
+			 * try again in 30 seconds AND then if renewAt's
+			 * value is after the lease expiration then
+			 * try to get a fresh lease for the interface
+			 */
+			if ((intf[closest].renewAt = pumpUptime() + 30) >
 			    intf[closest].leaseExpiration) {
 			    o = overrides;
 			    while (*o->intf.device &&
@@ -408,6 +442,8 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 			    if (!*o->intf.device) o = overrides;
 
 			    intf[closest].set &= ~PUMP_INTFINFO_HAS_LEASE;
+			    intf[closest].set |= PUMP_INTFINFO_NEEDS_NEWLEASE;
+
 
 			    if (pumpDhcpRun(intf[closest].device, 
 				  intf[closest].flags, 
@@ -415,6 +451,16 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 				  intf[closest].set & PUMP_NETINFO_HAS_HOSTNAME
 				    ? intf[closest].hostname : NULL,
 				  intf + closest, o)) {
+ 
+ 				    /* failed to get a new lease, so try
+				     * again in 30 seconds
+                                      */
+				    intf[closest].renewAt = pumpUptime() + 30;
+#if 0
+ 	/* ifdef this out since we now try more than once to get
+ 	 * a new lease and don't, therefore, want to remove the interface
+ 	 */
+ 
 				if (numInterfaces == 1) {
 				    callScript(o->script, PUMP_SCRIPT_DOWN,
 					       &intf[closest]);
@@ -426,7 +472,10 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 
 				intf[i] = intf[numInterfaces - 1];
 				numInterfaces--;
+#endif
 			    } else {
+				intf[closest].set &=
+					~PUMP_INTFINFO_NEEDS_NEWLEASE;
 				callScript(o->script, PUMP_SCRIPT_NEWLEASE,
 					   &intf[closest]);
                             }
@@ -444,7 +493,7 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 
 	if (select(sock + 1, &fds, NULL, NULL, 
 		   closest != -1 ? &tv : NULL) > 0) {
-	    conn = accept(sock, &addr, &addrLength);
+	    conn = accept(sock, (struct sockaddr *) &addr, &addrLength);
 
 	    if (read(conn, &cmd, sizeof(cmd)) != sizeof(cmd)) {
 		close(conn);
@@ -462,7 +511,7 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 
 		cmd.type = CMD_RESULT;
 		cmd.u.result = 0;
-		write(conn, &cmd, sizeof(cmd));
+		i = write(conn, &cmd, sizeof(cmd));
 		exit(0);
 
 	      case CMD_STARTIFACE:
@@ -523,8 +572,9 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 		    cmd.u.result = pumpDhcpRelease(intf + i);
 		    callScript(o->script, PUMP_SCRIPT_DOWN, intf + i);
 		    if (numInterfaces == 1) {
+                        int j;
 			cmd.type = CMD_RESULT;
-			write(conn, &cmd, sizeof(cmd));
+			j = write(conn, &cmd, sizeof(cmd));
 
 			syslog(LOG_INFO, "terminating as there are no "
 				"more devices under management");
@@ -574,7 +624,7 @@ static void runDaemon(int sock, char * configFile, struct pumpOverrideInfo * ove
 	    }
 
 	    if (cmd.type != CMD_STATUS) cmd.type = CMD_RESULT;
-	    write(conn, &cmd, sizeof(cmd));
+	    i = write(conn, &cmd, sizeof(cmd));
 
 	    close(conn);
 	}
@@ -752,13 +802,15 @@ int main (int argc, const char ** argv) {
     char * hostname = "";
     poptContext optCon;
     int rc;
+    int ret;
     int test = 0;
     int flags = 0;
-    int lease = 12;
+    int lease_hrs = 0;
+    int lease = 12*3600;
     int killDaemon = 0;
     int winId = 0;
     int release = 0, renew = 0, status = 0, lookupHostname = 0, nodns = 0;
-    int nogateway = 0;
+    int nogateway = 0, nobootp = 0;
     struct command cmd, response;
     char * configFile = "/etc/pump.conf";
     struct pumpOverrideInfo * overrides;
@@ -774,8 +826,10 @@ int main (int argc, const char ** argv) {
 			N_("iface") },
 	    { "kill", 'k', POPT_ARG_NONE, &killDaemon, 0,
 			N_("Kill daemon (and disable all interfaces)"), NULL },
-	    { "lease", 'l', POPT_ARG_INT, &lease, 0,
+	    { "lease", 'l', POPT_ARG_INT, &lease_hrs, 0,
 			N_("Lease time to request (in hours)"), N_("hours") },
+	    { "leasesecs", 'L', POPT_ARG_INT, &lease, 0,
+			N_("Lease time to request (in seconds)"), N_("seconds") },
 	    { "lookup-hostname", '\0', POPT_ARG_NONE, &lookupHostname, 0,
 			N_("Force lookup of hostname") },
 	    { "release", 'r', POPT_ARG_NONE, &release, 0,
@@ -788,6 +842,8 @@ int main (int argc, const char ** argv) {
 			N_("Don't update resolv.conf"), NULL },
 	    { "no-gateway", '\0', POPT_ARG_NONE, &nogateway, 0,
 			N_("Don't set a gateway for this interface"), NULL },
+	    { "no-bootp", '\0', POPT_ARG_NONE, &nobootp, 0,
+	                N_("Ignore non-DHCP BOOTP responses"), NULL },
 	    { "win-client-ident", '\0', POPT_ARG_NONE, &winId, 0,
 			N_("Set the client identifier to match Window's") },
 	    /*{ "test", 't', POPT_ARG_NONE, &test, 0,
@@ -831,6 +887,8 @@ int main (int argc, const char ** argv) {
 	flags |= PUMP_FLAG_FORCEHNLOOKUP;
     if (nodns)
 	overrides->flags |= OVERRIDE_FLAG_NODNS;
+    if (nobootp)
+	overrides->flags |= OVERRIDE_FLAG_NOBOOTP;
     if (nogateway)
 	overrides->flags |= OVERRIDE_FLAG_NOGATEWAY;
 
@@ -853,20 +911,23 @@ int main (int argc, const char ** argv) {
 	cmd.type = CMD_STARTIFACE;
 	strcpy(cmd.u.start.device, device);
 	cmd.u.start.flags = flags;
-	cmd.u.start.reqLease = lease * 60 * 60;
+	if(lease_hrs)
+		cmd.u.start.reqLease = lease_hrs * 60 * 60;
+	else
+		cmd.u.start.reqLease = lease;
 	strcpy(cmd.u.start.reqHostname, hostname);
     }
 
-    write(cont, &cmd, sizeof(cmd));
-    read(cont, &response, sizeof(response));
+    ret = write(cont, &cmd, sizeof(cmd));
+    ret = read(cont, &response, sizeof(response));
 
     if (response.type == CMD_RESULT && response.u.result &&
 	    cmd.type == CMD_STARTIFACE) {
 	cont = openControlSocket(configFile, overrides);
 	if (cont < 0) 
 	    exit(1);
-	write(cont, &cmd, sizeof(cmd));
-	read(cont, &response, sizeof(response));
+	ret = write(cont, &cmd, sizeof(cmd));
+	ret = read(cont, &response, sizeof(response));
     }
 
     if (response.type == CMD_RESULT) {
